import { Guid, Server } from 'org.crazydoctor.expressts';
import * as path from 'path';
import os from 'os';
import fs from 'fs';
import { spawn } from 'child_process';
import { Sources } from './sources/Sources';
import { CronJob } from 'cron';
import SHA256 from './util/SHA256';
import { Database } from 'sqlite3';

class ServerApp {
	public static SourcesUpdating = false;
	private static Server: Server | null = null;
	private static db: Database | null = null;

	private static sourcesUpdateTask(): void {
		if(ServerApp.SourcesUpdating)
			return;
		try {
			ServerApp.Server?.logInfo('[Cron] Sources update started...');
			ServerApp.SourcesUpdating = true;
			const update = spawn('node', [path.resolve(__dirname, 'util/UpdateSources.js')]);
	
			update.stdout.on('data', (data) => {
				console.log(data.toString());
			});

			update.stderr.on('data', (data) => {
				console.error(data.toString());
			});

			update.on('close', (code) => {
				ServerApp.SourcesUpdating = false;
				if (code === 0) {
					Sources.get();
					ServerApp.Server?.logInfo('[Cron] Sources update finished.');
				} else {
					ServerApp.Server?.logError('[Cron] Sources update failed.');
				}
			});

			update.on('error', (e) => {
				ServerApp.SourcesUpdating = false;
				ServerApp.Server?.logError('[Cron] Sources update failed.');
				console.error(e);
			});
		} catch (e) {
			ServerApp.SourcesUpdating = false;
			ServerApp.Server?.logError('[Cron] Sources update failed.');
			console.error(e);
		}
	}

	private static startCronTask(): void {
		const timeZone = 'Europe/Moscow';
		const cronSchedule = '0 3 * * *';
		const job = new CronJob(cronSchedule, ServerApp.sourcesUpdateTask, null, true, timeZone);
		job.start();
	}

	private static initDb(assetsDir: string): void {
		const dbDir = path.resolve(`${os.homedir()}/${assetsDir}/db`);
		if(!fs.existsSync(dbDir))
			fs.mkdirSync(dbDir);
		
		ServerApp.db = new Database(`${dbDir}/db.db`);

		ServerApp.db.run(`
			CREATE TABLE IF NOT EXISTS comments (
				id INTEGER PRIMARY KEY AUTOINCREMENT,
				class TEXT NOT NULL,
				property TEXT NOT NULL,
				comment TEXT,
				timestamp BIGINT
			)
		`);
	}

	public static getDb(): Database {
		return ServerApp.db!;
	}

	public static start(): void {

		const Config = JSON.parse(fs.readFileSync(path.resolve(__dirname, 'config.json'), { encoding: 'utf-8' }).toString());
		const ServerConfig = Config.server;
		const SourcesConfig = Config.sources;

		ServerApp.initDb(SourcesConfig.assetsDir);

		new Server({
			port: 3000,
			routesPath: path.resolve(__dirname, './routes'),
			middlewaresPath: path.resolve(__dirname, './middlewares'),
			viewsPath: path.resolve(__dirname, './views'),
			viewEngine: 'pug',
			options: {
				static: path.resolve(__dirname, './static'),
				sessionSecret: Guid.new(),
				adminPassword: SHA256.hash(ServerConfig.adminPassword)
			}
		}).init().then((server) => {
			server.start(() => {
				ServerApp.SourcesUpdating = true;
				Sources.get(() => {
					ServerApp.SourcesUpdating = false;
					ServerApp.startCronTask();
				});
			});
			ServerApp.Server = server;
		});
	}
}

ServerApp.start();

export default ServerApp;